/*******************************************************************************
 * Copyright (c) 2021, Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
 *
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *******************************************************************************/

//-----------------------------------------------------------------------------
/** @file  entity_helper.h */
//-----------------------------------------------------------------------------

#ifndef MANTLEAPI_TRAFFIC_ENTITY_HELPER_H
#define MANTLEAPI_TRAFFIC_ENTITY_HELPER_H

#include <MantleAPI/Traffic/i_entity.h>

#include <cmath>

namespace mantle_api
{

inline void SetSpeed(mantle_api::IEntity* entity, const units::velocity::meters_per_second_t &velocity)
{
  auto orientation = entity->GetOrientation();

  auto cos_elevation = units::math::cos(orientation.pitch);
  mantle_api::Vec3<units::velocity::meters_per_second_t> velocity_vector{velocity * units::math::cos(orientation.yaw) * cos_elevation,
                                    velocity * units::math::sin(orientation.yaw) * cos_elevation,
                                    velocity * -units::math::sin(orientation.pitch)};

  entity->SetVelocity(velocity_vector);
}

}  // namespace mantle_api

#endif  // MANTLEAPI_TRAFFIC_ENTITY_HELPER_H
